// Function Definitions
// Make a Sphere
Function MakeSphere
	// SphereParam[] = {Cx, Cy, Cz, R, SCL, SPN}
	
	// Center Position
	Cx  = SphereParam[0];
	Cy  = SphereParam[1];
	Cz  = SphereParam[2];
	// Radius
	R   = SphereParam[3];
	
	// Characterization Length
	SCL = SphereParam[4];
	
	// Arc Segmentation Number
	SPN = SphereParam[5];
	
	SphereCenter = newp;
	Point(SphereCenter) = {Cx, Cy, Cz, SCL};
	SphereTop = newp;
	Point(SphereTop) = {Cx, Cy, Cz+R, SCL};
	SphereBottom = newp;
	Point(SphereBottom) = {Cx, Cy, Cz-R, SCL};
	
	// Printf("Cx=%g, Cy=%g, Cz=%g", Cx, Cy, Cz);
	// Printf("R=%g, SCL=%g, SPN=%g", R, SCL, SPN);
	// Building XY-CirclePlane
	SphereXYPlanePoint[] = {};
	For i In {0:SPN-1}
		P = newp;
		Point(P) = {Cx+R, Cy, Cz, SCL};
		Rotate{ {0,0,1}, {Cx,Cy,Cz}, Pi*2/SPN*i } {Point{P};}
		SphereXYPlanePoint[] += P;
	EndFor
	
	SphereXYPlaneLine[] = {};
	SphereTopSideLine[] = {};
	SphereBottomSideLine[] = {};
	For i In{0:SPN-1}
		L = newl;
		Circle(L) = {SphereXYPlanePoint[i], SphereCenter, SphereXYPlanePoint[(i+1)%SPN]};
		SphereXYPlaneLine[] += L;
		
		L = newl;
		Circle(L) = {SphereTop, SphereCenter, SphereXYPlanePoint[i]};
		SphereTopSideLine[] += L;
		
		L = newl;
		Circle(L) = {SphereBottom, SphereCenter, SphereXYPlanePoint[i]};
		SphereBottomSideLine[] += L;
	EndFor
	
	// Building Surfaces
	SphereSurface[] = {};
	For i In {0:SPN-1}
		LL = newll;
		Line Loop(LL) = {SphereTopSideLine[i], SphereXYPlaneLine[i], -SphereTopSideLine[(i+1)%SPN]};
		S = news;
		Ruled Surface(S) = {LL} In Sphere {SphereCenter};
		SphereSurface[] += S;
		
		LL = newll;
		Line Loop(LL) = {SphereBottomSideLine[i], SphereXYPlaneLine[i], -SphereBottomSideLine[(i+1)%SPN]};
		S = news;
		Ruled Surface(S) = {LL} In Sphere {SphereCenter};
		SphereSurface[] += S;
	EndFor
Return

// Make a Box
Function MakeBox
	// BoxParam[] = {x0,y0,z0,x1,y1,z1,cl};
	x0 = BoxParam[0]; y0 = BoxParam[1]; z0 = BoxParam[2];
	x1 = BoxParam[3]; y1 = BoxParam[4]; z1 = BoxParam[5];
	PCL = BoxParam[6];
	BoxHeight = z1-z0;
	A = newp;
	Point(A) = {x0,y0,z0,PCL};
	B = newp;
	Point(B) = {x0,y1,z0,PCL};
	C = newp;
	Point(C) = {x1,y1,z0,PCL};
	D = newp;
	Point(D) = {x1,y0,z0,PCL};
	BoxBottomPoint[] = {A,B,C,D};
	BoxBottomLine[] = {};
	For i In {0:3}
		L = newl;
		Line(L) = {BoxBottomPoint[i], BoxBottomPoint[(i+1)%4]};
		BoxBottomLine[] += L;
	EndFor
	LL = newll;
	Line Loop(LL) = BoxBottomLine[];
	BoxBottomSurface = news;
	Ruled Surface(BoxBottomSurface) = {LL};
	
	BoxExtrusionResult[] = Extrude{0,0,BoxHeight}{Surface{BoxBottomSurface};Layers{Ceil(BoxHeight/PCL)};Recombine;};
	BoxTopSurface = BoxExtrusionResult[0];
	BoxVolume = BoxExtrusionResult[1];
	BoxSideSurface[] = BoxExtrusionResult[{2:5}];
	Printf("Making Box ...");
	Printf("BoxTopSurface = %g", BoxTopSurface);
Return

// Make a Box From Extrusion
Function MakeBoxFromExtrusion
	// BoxParam[] = {BottomSurface, BoxHeight, BoxLayerNum};
	BoxBottomSurface = BoxParam[0];
	BoxHeight = BoxParam[1];
	BoxLayerNum = BoxParam[2];
	BoxExtrusionResult[] = Extrude{0,0,BoxHeight}{Surface{BoxBottomSurface}; Layers{BoxLayerNum};Recombine;};
	BoxTopSurface = BoxExtrusionResult[0];
	BoxVolume = BoxExtrusionResult[1];
	BoxSideSurface[] = BoxExtrusionResult[{2:5}];
Return

